Struct AsyncService

pub struct AsyncService { /* private fields */ }
Available on crate feature async only.
High-level asynchronous SSDP service using tokio.

Handles incoming and outgoing searches using async, await, and the Tokio crate.

Implementations

Source

impl AsyncService

pub fn new() -> Result<Self, Error>

Create a new AsyncService, including its two UDP sockets

Errors

Can return a std::io::Error if any of the underlying socket calls fail.

Panics

Will panic if the internal mutex cannot be locked; that would indicate a bug in cotton-ssdp.

pub fn on_network_event(&self, event: &NetworkEvent) -> Result<(), Error>

Notify the AsyncService of a network interface change

Network interface changes can be obtained from cotton_netif::get_interfaces or cotton_netif::get_interfaces_async, filtering if desired – or, created manually.

Note that AsyncService will do nothing if it has no network interfaces to work with.

Panics

Will panic if the internal mutex cannot be locked; that would indicate a bug in cotton-ssdp.

pub fn subscribe<A>( &mut self, notification_type: A, ) -> impl Stream<Item = Notification>
where A: Into<String>,

Subscribe to SSDP notifications for a resource type.

Panics

Will panic if the internal mutex cannot be locked; that would indicate a bug in cotton-ssdp.

pub fn advertise<USN>( &mut self, unique_service_name: USN, advertisement: Advertisement, )
where USN: Into<String>,

Announce a new resource

And start responding to any searches matching it.

Panics

Will panic if the internal mutex cannot be locked; that would indicate a bug in cotton-ssdp.

pub fn deadvertise(&mut self, unique_service_name: &str)

Announce the disappearance of a resource

And stop responding to searches.

Panics

Will panic if the internal mutex cannot be locked; that would indicate a bug in cotton-ssdp.